==> cuf-components-1.1.41.jar: functions/array-functions.js <==
PW.Functions.register("filter", "string[]", ["string[]", "variable[]", "string[]", "string[]", "variable[]", "string[]", "variable[]",],
        function (array, singleFilterColumns, singleComparator, singleValues, multiFilterColumns, multiComparator, multipleValues) {
    return getFilteredArray(array, singleFilterColumns, singleComparator, singleValues, multiFilterColumns, multiComparator, multipleValues);
});

PW.Functions.register("filter", "integer[]", ["integer[]", "variable[]", "string[]", "string[]", "variable[]", "string[]", "variable[]",],
        function (array, singleFilterColumns, singleComparator, singleValues, multiFilterColumns, multiComparator, multipleValues) {
    return getFilteredArray(array, singleFilterColumns, singleComparator, singleValues, multiFilterColumns, multiComparator, multipleValues);
});

PW.Functions.register("filter", "float[]", ["float[]", "variable[]", "string[]", "string[]", "variable[]", "string[]", "variable[]",],
        function (array, singleFilterColumns, singleComparator, singleValues, multiFilterColumns, multiComparator, multipleValues) {
    return getFilteredArray(array, singleFilterColumns, singleComparator, singleValues, multiFilterColumns, multiComparator, multipleValues);
});

PW.Functions.register("filter", "date[]", ["date[]", "variable[]", "string[]", "string[]", "variable[]", "string[]", "variable[]",],
        function (array, singleFilterColumns, singleComparator, singleValues, multiFilterColumns, multiComparator, multipleValues) {
    return getFilteredArray(array, singleFilterColumns, singleComparator, singleValues, multiFilterColumns, multiComparator, multipleValues);
});

PW.Functions.register("filter", "datetime[]", ["datetime[]", "variable[]", "string[]", "string[]", "variable[]", "string[]", "variable[]",],
        function (array, singleFilterColumns, singleComparator, singleValues, multiFilterColumns, multiComparator, multipleValues) {
    return getFilteredArray(array, singleFilterColumns, singleComparator, singleValues, multiFilterColumns, multiComparator, multipleValues);
});

PW.Functions.register("boolean", "boolean[]", ["datetime[]", "variable[]", "string[]", "string[]", "variable[]", "string[]", "variable[]",],
        function (array, singleFilterColumns, singleComparator, singleValues, multiFilterColumns, multiComparator, multipleValues) {
    return getFilteredArray(array, singleFilterColumns, singleComparator, singleValues, multiFilterColumns, multiComparator, multipleValues);
});

PW.Functions.register("getSelectedRowNo", "integer", ["string"],
		function(tableId) {
	return getSelectedRowNumber(tableId);
});

PW.Functions.register("getSelectedRowIndex", "integer", ["string"],
    function(tableId) {
        return getSelectedRowNumber(tableId);
    });

function getSelectedRowNumber(tableId){
	try {
		var table = VariableSetService.get(tableId);
		var selectedRowNumber = VariableSetService.getRowIndex(table, VariableSetService.getSelected(table));
		return selectedRowNumber;
	} catch (err){
		Logger.error("Table with id: " + tableId + " not found.");
		return null;
	}
}

function getFilteredArray(array, singleFilterColumns, singleComparator, singleValues, multiFilterColumns, multiComparator, multipleValues){
    var rowsToFilterOut = filterColumnWithSingleValue(singleFilterColumns, singleComparator, singleValues, array.length);
    rowsToFilterOut = filterColumnWithMultipleValues(multiFilterColumns, multiComparator, multipleValues, rowsToFilterOut, array.length);
    var resultList = [];
    for (var i = 0; i < array.length; i++){
        if (!rowsToFilterOut[i]) resultList.push( array[i] );
    }
    return resultList;
}

function filterColumnWithSingleValue(singleFilterColumns, singleComparator, singleValues, length){
    var rowsToFilterOut = new Array(length);
    if (singleFilterColumns == null || singleFilterColumns.length == 0) return rowsToFilterOut;
    for (var i = 0; i < singleFilterColumns.length; i++)
    {
        var valueType = CUFCommon.getSingleType(singleFilterColumns[i].getType());
        var singleFilterValue = CUFCommon.stringToObject( singleValues[i], valueType );
        if (singleFilterColumns[i].getType().is("date[]")){
            singleFilterValue = PW.isEmpty(singleFilterValue) ? null : singleFilterValue.getTime();
        }
        for (var rowNumber = 0; rowNumber < length; rowNumber++)
        {
            var valueFromVariable = singleFilterColumns[i].getValue()[rowNumber];
            if (singleFilterColumns[i].getType().is("date[]")){
                valueFromVariable = PW.isEmpty(valueFromVariable) ? null : valueFromVariable.getTime();
            }
            if (!compareObjects[singleComparator[i]](valueFromVariable, singleFilterValue)){
                rowsToFilterOut[rowNumber] = true;
            }
        }
    }
    return rowsToFilterOut;
}

function filterColumnWithMultipleValues(multiFilterColumns, multiComparator, multipleValues, rowsToFilterOut){
    if (multiFilterColumns == null){
        return rowsToFilterOut;
    }
    for ( var i = 0; i < multiFilterColumns.length; i++)
    {
        if (multiFilterColumns[i].getType() != multipleValues[i].getType())
        {
            Logger.error("Columns in " + i + " row of function filter aren't the same type!");
        }
        for (var rowNumber = 0; rowNumber < rowsToFilterOut.length; rowNumber++)
        {
            var valueToCompare = multiFilterColumns[i].getValue()[rowNumber];
            var valueToCompareWith = multipleValues[i].getValue()[rowNumber];
            if (multiFilterColumns[i].getType().is("date[]")){
                valueToCompare = PW.isEmpty(valueToCompare) ? null : valueToCompare.getTime();
                valueToCompareWith = PW.isEmpty(valueToCompareWith)  ? null : valueToCompareWith.getTime();
            }
            if (!compareObjects[multiComparator[i]](valueToCompare, valueToCompareWith))
            {
                rowsToFilterOut[rowNumber] = true;
            }
        }
    }
    return rowsToFilterOut;
}

var compareObjects = {
    "=": function(cellValue, expectedValue) { return cellValue == expectedValue },
    "!=": function(cellValue, compareValue) { return cellValue != compareValue },
    "<": function(cellValue, compareValue) { return cellValue < compareValue },
    ">": function(cellValue, compareValue) { return cellValue > compareValue },
    "<=": function(cellValue, compareValue) { return cellValue <= compareValue },
    ">=": function(cellValue, compareValue) { return cellValue >= compareValue },
    "regex": function(cellValue, pattern) {
        let regExp = new RegExp(pattern, "u");
        return regExp.test(cellValue)
    }
};

PW.Functions.register("combineArrays", "string[]", ["string[]", "string[]"], function(array1, array2) {
    try
    {
        const maxLength = Math.max(array1.length, array2.length);
        let result = new Array(maxLength);

        for (let i = 0; i < maxLength; i++) 
        {
            if (i < array1.length && i < array2.length)
            {
                result[i] = array1[i] + array2[i];
            }
            else if (i < array1.length)
            {
                result[i] = array1[i];
            }
            else
            {
                result[i] = array2[i];
            }
        }

        return result;
    }
    catch (e)
    {
        Logger.error("Error in combineArrays: " + e);
        return [];
    }
});

==> cuf-components-1.1.43.jar: functions/array-functions.js <==
PW.Functions.register("filter", "string[]", ["string[]", "variable[]", "string[]", "string[]", "variable[]", "string[]", "variable[]",],
        function (array, singleFilterColumns, singleComparator, singleValues, multiFilterColumns, multiComparator, multipleValues) {
    return getFilteredArray(array, singleFilterColumns, singleComparator, singleValues, multiFilterColumns, multiComparator, multipleValues);
});

PW.Functions.register("filter", "integer[]", ["integer[]", "variable[]", "string[]", "string[]", "variable[]", "string[]", "variable[]",],
        function (array, singleFilterColumns, singleComparator, singleValues, multiFilterColumns, multiComparator, multipleValues) {
    return getFilteredArray(array, singleFilterColumns, singleComparator, singleValues, multiFilterColumns, multiComparator, multipleValues);
});

PW.Functions.register("filter", "float[]", ["float[]", "variable[]", "string[]", "string[]", "variable[]", "string[]", "variable[]",],
        function (array, singleFilterColumns, singleComparator, singleValues, multiFilterColumns, multiComparator, multipleValues) {
    return getFilteredArray(array, singleFilterColumns, singleComparator, singleValues, multiFilterColumns, multiComparator, multipleValues);
});

PW.Functions.register("filter", "date[]", ["date[]", "variable[]", "string[]", "string[]", "variable[]", "string[]", "variable[]",],
        function (array, singleFilterColumns, singleComparator, singleValues, multiFilterColumns, multiComparator, multipleValues) {
    return getFilteredArray(array, singleFilterColumns, singleComparator, singleValues, multiFilterColumns, multiComparator, multipleValues);
});

PW.Functions.register("filter", "datetime[]", ["datetime[]", "variable[]", "string[]", "string[]", "variable[]", "string[]", "variable[]",],
        function (array, singleFilterColumns, singleComparator, singleValues, multiFilterColumns, multiComparator, multipleValues) {
    return getFilteredArray(array, singleFilterColumns, singleComparator, singleValues, multiFilterColumns, multiComparator, multipleValues);
});

PW.Functions.register("boolean", "boolean[]", ["datetime[]", "variable[]", "string[]", "string[]", "variable[]", "string[]", "variable[]",],
        function (array, singleFilterColumns, singleComparator, singleValues, multiFilterColumns, multiComparator, multipleValues) {
    return getFilteredArray(array, singleFilterColumns, singleComparator, singleValues, multiFilterColumns, multiComparator, multipleValues);
});

PW.Functions.register("getSelectedRowNo", "integer", ["string"],
		function(tableId) {
	return getSelectedRowNumber(tableId);
});

PW.Functions.register("getSelectedRowIndex", "integer", ["string"],
    function(tableId) {
        return getSelectedRowNumber(tableId);
    });

PW.Functions.register("getSelectedColumnId", "string", ["string"],
    function (tableId) {
        const table = VariableSetService.get(tableId);
        return getSelectedColumnId(table);
    });

PW.Functions.register("getSelectedCellValue", "string", ["string"],
    function(tableId) {
        try {
            const table = VariableSetService.get(tableId);
            const columnId = getSelectedColumnId(table);

            if (columnId !== "") {
                const value = table.getSelectionModel().selection.record.data[columnId];
                return PW.Types.get("string").convert(value);
            }
        } catch (error) {
            Logger.error("Error fetching selected cell value: " + error);
        }

        return "";
    });

function getSelectedColumnId(table) {
    try {
        // FIXME: wywołanie oficjalnego API (PWFL-12808)
        const selectionModel = table.getSelectionModel();

        if (selectionModel.hasSelection) {
            const columnIndex = selectionModel.selection.column;
            return table.headerCt.getGridColumns()[columnIndex].dataIndex;
        }
    } catch (error) {
        Logger.error("Error fetching selected column ID: " + error);
    }

    return "";
}

function getSelectedRowNumber(tableId){
	try {
		var table = VariableSetService.get(tableId);
		var selectedRowNumber = VariableSetService.getRowIndex(table, VariableSetService.getSelected(table));
		return selectedRowNumber;
	} catch (err){
		Logger.error("Table with id: " + tableId + " not found.");
		return null;
	}
}

function getFilteredArray(array, singleFilterColumns, singleComparator, singleValues, multiFilterColumns, multiComparator, multipleValues){
    var rowsToFilterOut = filterColumnWithSingleValue(singleFilterColumns, singleComparator, singleValues, array.length);
    rowsToFilterOut = filterColumnWithMultipleValues(multiFilterColumns, multiComparator, multipleValues, rowsToFilterOut, array.length);
    var resultList = [];
    for (var i = 0; i < array.length; i++){
        if (!rowsToFilterOut[i]) resultList.push( array[i] );
    }
    return resultList;
}

function filterColumnWithSingleValue(singleFilterColumns, singleComparator, singleValues, length){
    var rowsToFilterOut = new Array(length);
    if (singleFilterColumns == null || singleFilterColumns.length == 0) return rowsToFilterOut;
    for (var i = 0; i < singleFilterColumns.length; i++)
    {
        var valueType = CUFCommon.getSingleType(singleFilterColumns[i].getType());
        var singleFilterValue = CUFCommon.stringToObject( singleValues[i], valueType );
        if (singleFilterColumns[i].getType().is("date[]")){
            singleFilterValue = PW.isEmpty(singleFilterValue) ? null : singleFilterValue.getTime();
        }
        for (var rowNumber = 0; rowNumber < length; rowNumber++)
        {
            var valueFromVariable = singleFilterColumns[i].getValue()[rowNumber];
            if (singleFilterColumns[i].getType().is("date[]")){
                valueFromVariable = PW.isEmpty(valueFromVariable) ? null : valueFromVariable.getTime();
            }
            if (!compareObjects[singleComparator[i]](valueFromVariable, singleFilterValue)){
                rowsToFilterOut[rowNumber] = true;
            }
        }
    }
    return rowsToFilterOut;
}

function filterColumnWithMultipleValues(multiFilterColumns, multiComparator, multipleValues, rowsToFilterOut){
    if (multiFilterColumns == null){
        return rowsToFilterOut;
    }
    for ( var i = 0; i < multiFilterColumns.length; i++)
    {
        if (multiFilterColumns[i].getType() != multipleValues[i].getType())
        {
            Logger.error("Columns in " + i + " row of function filter aren't the same type!");
        }
        for (var rowNumber = 0; rowNumber < rowsToFilterOut.length; rowNumber++)
        {
            var valueToCompare = multiFilterColumns[i].getValue()[rowNumber];
            var valueToCompareWith = multipleValues[i].getValue()[rowNumber];
            if (multiFilterColumns[i].getType().is("date[]")){
                valueToCompare = PW.isEmpty(valueToCompare) ? null : valueToCompare.getTime();
                valueToCompareWith = PW.isEmpty(valueToCompareWith)  ? null : valueToCompareWith.getTime();
            }
            if (!compareObjects[multiComparator[i]](valueToCompare, valueToCompareWith))
            {
                rowsToFilterOut[rowNumber] = true;
            }
        }
    }
    return rowsToFilterOut;
}

var compareObjects = {
    "=": function(cellValue, expectedValue) { return cellValue == expectedValue },
    "!=": function(cellValue, compareValue) { return cellValue != compareValue },
    "<": function(cellValue, compareValue) { return cellValue < compareValue },
    ">": function(cellValue, compareValue) { return cellValue > compareValue },
    "<=": function(cellValue, compareValue) { return cellValue <= compareValue },
    ">=": function(cellValue, compareValue) { return cellValue >= compareValue },
    "regex": function(cellValue, pattern) {
        let regExp = new RegExp(pattern, "u");
        return regExp.test(cellValue)
    }
};

PW.Functions.register("combineArrays", "string[]", ["string[]", "string[]"], function(array1, array2) {
    try
    {
        const maxLength = Math.max(array1.length, array2.length);
        let result = new Array(maxLength);

        for (let i = 0; i < maxLength; i++) 
        {
            if (i < array1.length && i < array2.length)
            {
                result[i] = array1[i] + array2[i];
            }
            else if (i < array1.length)
            {
                result[i] = array1[i];
            }
            else
            {
                result[i] = array2[i];
            }
        }

        return result;
    }
    catch (e)
    {
        Logger.error("Error in combineArrays: " + e);
        return [];
    }
});

PW.Functions.register("findIndex", "integer", ["string[]", "string"], function (array, value) {
    return findIndexInArray(array, value, 0);
});

PW.Functions.register("findIndex", "integer", ["string[]", "string", "integer"], function (array, value, start) {
    return findIndexInArray(array, value, start);
});

PW.Functions.register("findIndex", "integer", ["integer[]", "integer"], function (array, value) {
    return findIndexInArray(array, value, 0);
});

PW.Functions.register("findIndex", "integer", ["integer[]", "integer", "integer"], function (array, value, start) {
    return findIndexInArray(array, value, start);
});

PW.Functions.register("findIndex", "integer", ["float[]", "float"], function (array, value) {
    return findIndexInArray(array, value, 0);
});

PW.Functions.register("findIndex", "integer", ["float[]", "float", "integer"], function (array, value, start) {
    return findIndexInArray(array, value, start);
});

PW.Functions.register("findIndex", "integer", ["date[]", "date"], function (array, value) {
    return findIndexInArray(array, value, 0);
});

PW.Functions.register("findIndex", "integer", ["date[]", "date", "integer"], function (array, value, start) {
    return findIndexInArray(array, value, start);
});

PW.Functions.register("findIndex", "integer", ["datetime[]", "datetime"], function (array, value) {
    return findIndexInArray(array, value, 0);
});

PW.Functions.register("findIndex", "integer", ["datetime[]", "datetime", "integer"], function (array, value, start) {
    return findIndexInArray(array, value, start);
});

PW.Functions.register("findIndex", "integer", ["boolean[]", "boolean"], function (array, value) {
    return findIndexInArray(array, value, 0);
});

PW.Functions.register("findIndex", "integer", ["boolean[]", "boolean", "integer"], function (array, value, start) {
    return findIndexInArray(array, value, start);
});

function findIndexInArray(array, value, start) {
    try {
        if (!array || start < 0 || start >= array.length) {
            return -1;
        }
        for (let i = start; i < array.length; i++) {
            if (value instanceof Date && array[i] instanceof Date) {
                if (value.getTime() === array[i].getTime()) {
                    return i;
                }
            } else {
                if (value === array[i]) {
                    return i;
                }
            }
        }
        return -1;
    } catch (e) {
        Logger.error("Error in findIndex: " + e);
        return -1;
    }
}
